function[Q,R] = gram_schmidt(A)
% Computes a QR factorization for an m x n matrix A (m >= n).
% Assumes that A has full rank (i.e., does not pivot)

[m,n] = size(A);
assert( m >= n );

Q = zeros([m n]);
R = zeros([n n]);

R(1,1) = norm(A(:,1));
Q(:,1) = A(:,1)/R(1,1);

for qn = 1:(n-1)

  R(1:qn,qn+1) = Q(:,1:qn)' * A(:,qn+1);
  v = A(:,qn+1) - Q(:,1:qn) * R(1:qn,qn+1);

  R(qn+1,qn+1) = norm(v);
  Q(:,qn+1) = v/R(qn+1,qn+1);

end
