function[Q,R] = mod_gram_schmidt(A)
% Computes a QR factorization for an m x n matrix A (m >= n).
% Assumes that A has full rank (i.e., does not pivot)

[m,n] = size(A);
assert( m >= n );

Q = zeros([m n]);
R = zeros([n n]);

R(1,1) = norm(A(:,1));
Q(:,1) = A(:,1)/R(1,1);

R(1,2:n) = Q(:,1)'*A(:,2:n);
A(:,2:n) = A(:,2:n) - Q(:,1)*R(1,2:n);

for qn = 1:(n-1)

  R(qn+1,qn+1) = norm(A(:,qn+1));
  Q(:,qn+1) = A(:,qn+1)/R(qn+1,qn+1);

  R(qn+1, (qn+2):n) = Q(:,(qn+1))'*A(:,(qn+2):n);
  A(:,(qn+2):n) = A(:,(qn+2):n) - Q(:,(qn+1))*R(qn+1, (qn+2):n);

end
